#include "hiscore.h"

#include "game.h"
#include "asmroutines.h"
#include "render.h"
#include "ui.h"
#include "z_string.h"
#include "y_disk_asm.h"

const u8 minAscVal = 0x40; // @ translated to SPACE
const u8 maxAscVal = 0x5A; // Z
char *curHSString; // Pointer to trigram of current game 
u8 curHSPos;       // Position of modifiable char (0-2)

void initHiScore ()
{
    hiscore [0].time = 50;
    (hiscore[0].trigram) [0] = 'C';
    (hiscore[0].trigram) [1] = 'B';
    (hiscore[0].trigram) [2] = 'A';
    (hiscore[0].trigram) [3] = 0;
    //cpct_memcpy ( &(hiscore[0].trigram), "CBA", 4);
    hiscore [1].time = 200;
    (hiscore[1].trigram) [0] = 'G';
    (hiscore[1].trigram) [1] = 'C';
    (hiscore[1].trigram) [2] = 'R';
    (hiscore[1].trigram) [3] = 0;
//    cpct_memcpy ( &(hiscore[1].trigram), "GCR", 4);
    hiscore [2].time = 500;
    (hiscore[2].trigram) [0] = 'X';
    (hiscore[2].trigram) [1] = 'J';
    (hiscore[2].trigram) [2] = 'T';
    (hiscore[2].trigram) [3] = 0;
//    cpct_memcpy ( &(hiscore[2].trigram), "XJT", 4);
    hiscore [3].time = 999;
    (hiscore[3].trigram) [0] = 'S';
    (hiscore[3].trigram) [1] = 'G';
    (hiscore[3].trigram) [2] = 'A';
    (hiscore[3].trigram) [3] = 0;
//    cpct_memcpy ( &(hiscore[3].trigram), "SGA", 4);
}

u8 checkHighScore (u8 game, u16 time)
{
    u8 rc = 1;
    if (time < hiscore[game].time)
        hiscore[game].time = time;
    else
        rc = 0;
    return rc;
}

char * getHiscoreString (u8 game)
{
    getStringFromWord (strBuffer, hiscore[game].time); // Need 5 chars for word, but max 3 so first 2 are space
    strBuffer[0] = '1'+game;
    strBuffer[5] = ' ';
    strBuffer[6] = ' ';
    strBuffer[7] = hiscore[game].trigram[0];
    strBuffer[8] = hiscore[game].trigram[1];
    strBuffer[9] = hiscore[game].trigram[2];
    strBuffer[10] = 0;

    return &strBuffer[0];
}

u16 getHiscoreValue (u8 game)
{
    return hiscore[game].time;
}

void displayHSText ()
{
    pvmem = getScreenLineAdress (24) + 62;
    cpct_drawStringM1(curHSString,pvmem);
}

void invertHSPos ()
{
    pvmem = getScreenLineAdress (24) + 62 + (curHSPos <<1);
    invertSquare (pvmem);
}

void initHSInput (u8 game)
{
    curHSPos = 0; // Init of cursor
    curHSString = &(hiscore[game].trigram[0]); // Pointing the correct string in hiscore table

    cpct_setDrawCharM1(2, 3);
    pvmem = getScreenLineAdress (24) + 4;
    cpct_drawStringM1(dispHiscoreStr,pvmem); // The "You got a hi score text"

    displayHSText ();
    invertHSPos (); // Inverse first char
}

void prevHSChar ( )
{
    invertHSPos ();
    curHSString [curHSPos] --;
    if ( curHSString [curHSPos] == minAscVal )
        curHSString [curHSPos] = ' ';
    else if ( curHSString [curHSPos] < minAscVal )
        curHSString [curHSPos] = maxAscVal;
    displayHSText ();
    invertHSPos();
}

void nextHSChar ( )
{
    invertHSPos ();
    curHSString [curHSPos] ++;
    if ( curHSString [curHSPos] > maxAscVal )
        curHSString [curHSPos] = ' ';
    else if ( curHSString [curHSPos] < minAscVal )
        curHSString [curHSPos] = minAscVal + 1;
    displayHSText ();
    invertHSPos ();
}

void prevHSPos ( )
{
    if (curHSPos == 0) return;
    invertHSPos ();
    curHSPos --;
    invertHSPos ();
}

void nextHSPos ( )
{
    if (curHSPos == 2) return;
    invertHSPos ();
    curHSPos ++;
    invertHSPos ();
}

void endHSDisplay ()
{
    restoreFirmware();
    SaveHiScore();
    disableFirmware();
    displayExit ();
}
