#include "pointer.h"

#include "grid.h"
#include "game.h"
#include "render.h"

const unsigned char G_Pointeur [4][36] =
{
   { 
	0x00, 0xF0, 0x33, 0x84, 0xFF, 0x00, 
	0x00, 0xE1, 0x77, 0x08, 0xFF, 0x00, 
	0x00, 0xF0, 0x77, 0x08, 0xFF, 0x00, 
	0x00, 0xB4, 0x33, 0x84, 0xFF, 0x00, 
	0x00, 0x96, 0x33, 0xC0, 0xFF, 0x00, 
	0x66, 0x09, 0x33, 0xC0, 0xFF, 0x00},
   { 
	0x88, 0x70, 0x11, 0xC2, 0xFF, 0x00, 
	0x88, 0x70, 0x33, 0x0C, 0xFF, 0x00, 
	0x88, 0x70, 0x33, 0x84, 0xFF, 0x00, 
	0x88, 0x52, 0x11, 0xC2, 0xFF, 0x00, 
	0x88, 0x43, 0x11, 0xE0, 0xFF, 0x00, 
	0xBB, 0x04, 0x11, 0x68, 0xFF, 0x00},
   { 
	0xCC, 0x30, 0x00, 0xE1, 0xFF, 0x00, 
	0xCC, 0x30, 0x11, 0x86, 0xFF, 0x00, 
	0xCC, 0x30, 0x11, 0xC2, 0xFF, 0x00, 
	0xCC, 0x21, 0x00, 0xE1, 0xFF, 0x00, 
	0xCC, 0x21, 0x00, 0x78, 0xFF, 0x00, 
	0xDD, 0x02, 0x88, 0x34, 0xFF, 0x00},
   { 
	0xEE, 0x10, 0x00, 0xF0, 0x77, 0x08, 
	0xEE, 0x10, 0x00, 0xC3, 0xFF, 0x00, 
	0xEE, 0x10, 0x00, 0xE1, 0xFF, 0x00, 
	0xEE, 0x10, 0x00, 0x78, 0x77, 0x08, 
	0xEE, 0x10, 0x00, 0x3C, 0x77, 0x80, 
	0xEE, 0x01, 0xCC, 0x12, 0x77, 0x80}
};

#define pointerWidth 3
#define pointerHeight 6

u8 pointerX, pointerY;  // Byte position on screen from top/left
i8 nbCurX,              // Toggle the pointer cursor between 4 X positions
   pointerSpeed;        // Fast (x2) or slow (x1) movement

u8 pointeurBackGround [ pointerWidth * pointerHeight ]; // To save background of pointer

void initPointer ()
{
    pointerX = 40;
    pointerY = 100;
    nbCurX = 0;
    initPointerSpeed();
}

void togglePointerSpeed ()
{
    pointerSpeed = ( pointerSpeed == 1 ? 2 : 1);
}

void initPointerSpeed()
{
   pointerSpeed = 1;
}

void clearPointerScreen ()
{
   pvmem = cpct_getScreenPtr (CPCT_VMEM_START, pointerX,pointerY);
   cpct_drawSprite (pointeurBackGround, pvmem, pointerWidth, pointerHeight);
}

void savePointerScreen ()
{
   pvmem = cpct_getScreenPtr (CPCT_VMEM_START, pointerX,pointerY);
   cpct_getScreenToSprite (pvmem, pointeurBackGround, pointerWidth, pointerHeight);
}

void printPointerScreen ()
{
   pvmem = cpct_getScreenPtr (CPCT_VMEM_START, pointerX,pointerY);
   cpct_drawSpriteMasked (G_Pointeur[nbCurX], pvmem, pointerWidth, pointerHeight);
}

u8 getGridFromPointer (u8 *gridX, u8 *gridY)
{
   i8 valX, valY;
   u8 offX, offY;
   getGridOffset (&offX,&offY);
   // 
   valX = (pointerX - offX) >> 1; // Divided by 2
   valY = (pointerY - (offY << 3)) >> 3; // Divided by 8
   if (valX > gGridWidth - 1 || valY > gGridHeight - 1
      || valX < 0 || valY <0 )
      return 0;
   else 
   {
      *gridX = valX;
      *gridY = valY;
      return 1;
   }
}

void movePointer (i8 dx, i8 dy)
{
   if ( dx == 1 ) 
   {
      nbCurX += pointerSpeed;
      if (nbCurX > 3) // need to change X column
      {
         if ( pointerX >= 80 - pointerWidth ) // Last column ?
            nbCurX = 3, dx = 0; // Use last sprite on last column
         else
            nbCurX -= 4; // Next sprite to display in the next column
      }
      else
         dx = 0; // Don't change current byte but switch sprite
   } else if ( dx == -1 ) 
   {
      nbCurX -= pointerSpeed;
      if (nbCurX < 0)
      {
         if ( pointerX == 0 ) // Last column ?
         {
            nbCurX = 0; dx = 0; // Use last ptr on last column
         }
         else
            nbCurX += 4; // Next sprite to display in the next column
      }
      else
         dx = 0; // Don't change current byte but switch sprite
   }
   
   if ( dy == 1 )
   {
      dy = pointerSpeed;
      if ( pointerY + dy >= 200 - pointerHeight)
         dy = 0;
   } else if ( dy == -1 )
   {
      dy = -pointerSpeed;
      if ( pointerY + dy < 0)
         dy = 0;
   }

   clearPointerScreen ();
   pointerX += dx;
   pointerY += dy;
   savePointerScreen ();
   printPointerScreen ();
}
