#include "render.h"

#include "grid.h"
#include "sprites.h"
#include "pointer.h"

u8* lgAdress [25] ;
u8 lgOffsetX, lgOffsetY,gShowBomb;

const u8 G_palette [4] = {HW_WHITE,HW_BLACK,HW_BRIGHT_WHITE,HW_BRIGHT_RED};
const u8 G_GreyPpalette [4] = {FW_WHITE,FW_WHITE,FW_WHITE,FW_WHITE};

void initRender ()
{
   u8 *offset; 
   u8 i;

   cpct_setPalette (G_palette,4);
   cpct_setBorder (HW_CYAN);

   offset = CPCT_VMEM_START;
   for (i=0; i<25 ; i++)
   {
      lgAdress[i] = offset;
      offset += 0x50;
   }

   lgOffsetX = lgOffsetY = gShowBomb = 0;
}

void setGridOffset ()
{
   lgOffsetX = 32 - gGridWidth;
   lgOffsetY = 12 - gGridHeight / 2;
}

void getGridOffset (u8 *offsetX, u8 *offsetY)
{
   *offsetX = lgOffsetX;
   *offsetY = lgOffsetY;
}

void printSprite (u8 *vmem, u8 sprnum)
{
   u8 *sproffset; 
   u8 i;
   sproffset = (u8*) G_Sprites [sprnum];
   for (i=0;i<8;i++)
   {
      vmem[0] = *sproffset++;
      vmem[1] = *sproffset++;
      vmem += 0x800;
   }
}

u8 * getScreenLineAdress (u8 gridY)
{
   return lgAdress [gridY];
}

u8 * getGridAdress (u8 gridX, u8 gridY)
{
   return lgAdress [gridY+lgOffsetY] + lgOffsetX + (gridX <<1);
}

void printCell (u16 offset, u8 digVal, u8* pvmem)
{
   u8 forceShow, val, sprite;
   forceShow = gShowBomb;
   sprite = SPR2_TILE;  // Not digged
   if (!gShowBomb)
   {
      switch ( digVal ) 
      {
         case 1: // Digged, display the case
            forceShow = 1; break;
         case 2: // flagged
            sprite = SPR2_FLAG; break;
         case 3: // ?
            sprite = SPR2_QMARK; break;
         default: // Not digged
            break;
      }
   }
   
   if ( forceShow )
   {
      val = bombTable [offset];
      if ( val & 128 ) // Bomb
      {
         sprite = SPR2_BOMB; // Display the bomb or...
         if ( digVal == 1) 
            sprite = SPR2_EXPLOSION;  // You exploded here
         else if ( digVal == 2) 
            sprite = SPR2_FLAG;       // Youput a correct flag
      }
      else  // not a bomb
      {
         if (digVal == 1 || digVal == 3) // Digged or QMark
            sprite = SPR2_C0 + val;      // Number of bombs around
         else if (digVal == 2)           // Wrongly flagged
            sprite = SPR2_BADBOMB;
      }
   }
   printSprite(pvmem, sprite);
}

void printGrid ()
{
   u8 * pvmem;
   u16 offset;
   i8 x, y, digVal;

   offset = 0;
   for (y=0; y<gGridHeight; y++)
   {
      pvmem = getGridAdress (0,y);
      for (x=0; x<gGridWidth; x++)
      {
         digVal = digTable [offset];
         printCell (offset, digVal, pvmem);
         offset++;
         pvmem ++;pvmem ++;
      }
   }
}

void clearSquare (u8* pvmem)
{
   u8 y;
   for (y=0; y<8; y++)
   {
      pvmem[0] = 0;
      pvmem[1] = 0;
      pvmem += 0x800;
   }
}

void invertSquare (u8* pvmem)
{
   u8 y;
   for (y=0; y<8; y++)
   {
      pvmem[0] = pvmem[0] ^ 0xFF;
      pvmem[1] = pvmem[1] ^ 0xFF;
      pvmem += 0x800;
   }
}
