#include "ui.h"

#include "clock.h"
#include "game.h"
#include "render.h"
#include "grid.h"
#include "pointer.h"
#include "sprites.h"
#include "music.h"
#include "hiscore.h"
#include "z_string.h"

#include "asmroutines.h"

// Locals
u8  startExploding, menuAnimPos;
i8  menuAnimDirection;
u16 menuAnimFrameTick;

// Exploding bombs...
#define NBEXPL 15
u8 *expldPos [NBEXPL], nextExplosion [NBEXPL], expldFrame [NBEXPL];
u8 xBMin, yBMin, xBSize, yBSize, nbBExpl, bombExplTick;

void resetValues()
{
    menuAnimDirection = -1; // To start left 
    menuAnimFrameTick = 41; // with 5s delay 
    menuAnimPos       =  0; // 
    startExploding    =  0; // Starts menu explosions after 5s
    initBombExpl ( 18, 1, 41, 6, 6 ); // Init with logo dimensions
}

void initBombExpl (u8 xmin, u8 ymin, u8 sizex, u8 sizey, u8 nbBomb)
{
    u8 rndTimer ;
    xBMin = xmin;
    yBMin = ymin;
    xBSize = sizex;
    yBSize = sizey;
    nbBExpl = nbBomb;
    bombExplTick = 0;
    
    for (int i=0; i<nbBExpl; i++)
    {
        rndTimer = cpct_getRandom_mxor_u8() & 0x08;
        if ( !i ) rndTimer = 1; // First one blows asap
        while (rndTimer >= 6) 
            rndTimer -= 6;
        nextExplosion [i] =  i >> 2 + rndTimer;
        expldFrame  [i]   =  0; // Restet current sprite
    }
}

void initBombExplEnd ()
{
    // Inits exploding bombswith grid dimensions
    u8 xOff,yOff;
    getGridOffset (&xOff, &yOff);
    initBombExpl (xOff, yOff, gGridWidth * 2, gGridHeight, 14);
}


void updateBombExpl ()
{
    u8 x, y, rndTimer, i;

    for (i=0; i<nbBExpl ; i++)
    {
        if ( !nextExplosion [i] )
        {
            if ( expldFrame [i] == 0)
            {
                // Randomly choose a position
                x = cpct_getRandom_mxor_u8();
                while ( x >= xBSize )
                    x -= xBSize;
                y = cpct_getRandom_mxor_u8();
                while ( y >= yBSize )
                    y -= yBSize;

                expldPos[i] = getScreenLineAdress ( y + yBMin ) + xBMin + x;
            }
            if ( bombExplTick  == 3)
            {
                bombExplTick = 0;
                cpct_drawSpriteBlended (expldPos[i], 8, 2, G_MenuBomb[expldFrame[i]]);
                expldFrame[i]++;
                if (expldFrame[i] == 6)
                {
                    rndTimer = cpct_getRandom_mxor_u8() & 0x10;
                    expldFrame[i] = 0;
                    nextExplosion [i] = rndTimer;
                }
            }
            else
                bombExplTick++;
        }
        else
            nextExplosion [i]--;
    }
}


void initUI()
{
    cpct_setBlendMode (CPCT_BLEND_XOR);
    resetValues();
}


void clearScreenAnimation ()
{
    u8 *pvmem2;
    u8 x, li;
    for (x = 0; x<32; x++)
    {
        pvmem = getScreenLineAdress(0) + (x << 1); // First Line from left to right
        pvmem2 = getScreenLineAdress(24) + 78 - (x << 1); // Last Line from right to left
        li = 0;
        while ( li <= x) 
        {
            if (li <= 24 )
            {
                clearSquare(pvmem);
                clearSquare(pvmem2);
            }
            pvmem += 0x50 - 2;
            pvmem2 += (-0x50 + 2);
            li++;
        } 
    }
}

/*
void clearScreenAnimationLine ()
{
    u8 *pvmemLg1, *pvmemLg2, *pvmem1, *pvmem2;
    i8 x, x2;
    pvmemLg1 = CPCT_VMEM_START;
    pvmemLg2 = CPCT_VMEM_START + 24 * 0x50;
    for (x = 0; x<13; x++)
    {
        pvmem1 = pvmemLg1;
        pvmem2 = pvmemLg2;

        for (x2=0; x2 < 40; x2++)
        {
            clearSquare(pvmem1);
            pvmem1 ++;pvmem1 ++;
            clearSquare(pvmem2);
            pvmem2 ++;pvmem2 ++;
        }
        pvmemLg1 += 0x50;
        pvmemLg2 -= 0x50;
    }
}
*/

void getStringFromInt(char *Buff , u8 val)
{
    u8 tmpVal, posChar;

    Buff[0] = ' ';
    Buff[1] = ' ';
    Buff[2] = ' ';
    Buff[3] = 0;
    posChar = 2;

    do
    {
        tmpVal = val / 10;
        Buff[posChar--] = val - 10 * tmpVal + '0';
        val = tmpVal;
    } while (tmpVal > 0);
}

void displayAnimMenu (u8 pos)
{
    if ( menuAnimDirection == 1)
    {
        pvmem = getScreenLineAdress (19);
        asm_mvTextLeft();
        pvmem = getScreenLineAdress (19) + 78;
        cpct_drawCharM1_f(pvmem, 2,0,gKeys[39+pos]);

        pvmem = getScreenLineAdress (21) + 79;
        asm_mvTextRight();
        pvmem = getScreenLineAdress (21);
        cpct_drawCharM1_f(pvmem, 2,0,gHint[40-pos]);

        pvmem = getScreenLineAdress (24);
        asm_mvTextLeft();
        pvmem = getScreenLineAdress (24) + 78;
        cpct_drawCharM1_f(pvmem, 1,0,gCopy[39+pos]);
    }
    else
    {
        pvmem = getScreenLineAdress (19) + 79;
        asm_mvTextRight();
        pvmem = getScreenLineAdress (19);
        cpct_drawCharM1_f(pvmem, 2,0,gKeys[pos]);

        pvmem = getScreenLineAdress (21);
        asm_mvTextLeft();
        pvmem = getScreenLineAdress (21) + 78;
        cpct_drawCharM1_f(pvmem, 2,0,gHint[79-pos]);

        pvmem = getScreenLineAdress (24) + 79;
        asm_mvTextRight();
        pvmem = getScreenLineAdress (24);
        cpct_drawCharM1_f(pvmem, 1,0,gCopy[pos]);
    }
}

void displayMusicOption ( u8 curOption )
{
    const char * str [3] = {strOn,strMenu,strOff};
    pvmem = getScreenLineAdress (14) + 44;
    cpct_drawStringM1(str[curOption],pvmem);
}

void titleScreen()
{
    char RememberChar;
    u8 * ptrChar;

    pvmem = getScreenLineAdress (2) + 20;
    cpct_drawSprite (G_Logo, pvmem, 38, 32);

    cpct_setDrawCharM1(1, 0);
    pvmem = getScreenLineAdress (8) + 2;
    cpct_drawStringM1(menu_game,pvmem);

    pvmem = getScreenLineAdress (10) + 6;
    cpct_drawStringM1(menu_auto,pvmem);

    pvmem = getScreenLineAdress (12) + 6;
    cpct_drawStringM1(menu_qmrk,pvmem);

    pvmem = getScreenLineAdress (14) + 6;
    cpct_drawStringM1(menu_musc,pvmem);

    pvmem = getScreenLineAdress (16) + 6;
    cpct_drawStringM1(menu_strt,pvmem);

    pvmem = getScreenLineAdress (8) + 60;
    cpct_setDrawCharM1(2, 1);
    cpct_drawStringM1(menu_hisc,pvmem);
    pvmem += 0x50;
    cpct_setDrawCharM1(0, 1);
    cpct_drawStringM1(menu_spac,pvmem);
    pvmem += 0x50;
    cpct_setDrawCharM1(1, 3);
    cpct_drawStringM1(menu_hstb,pvmem);
    pvmem += 0x50;
    cpct_setDrawCharM1(0, 3);
    cpct_drawStringM1(menu_spac,pvmem);
    pvmem += 0x50;
    cpct_setDrawCharM1(2, 3);
    cpct_drawStringM1(getHiscoreString(0),pvmem);
    pvmem += 0x50;
    cpct_drawStringM1(getHiscoreString(1),pvmem);
    pvmem += 0x50;
    cpct_drawStringM1(getHiscoreString(2),pvmem);
    pvmem += 0x50;
    cpct_drawStringM1(getHiscoreString(3),pvmem);
    pvmem += 0x50;
    cpct_setDrawCharM1(0, 3);
    cpct_drawStringM1(menu_spac,pvmem);

    displayMenuValues ();
    displayMusicOption (getMusicOption());

    // InitUI
    resetValues(); 

    ptrChar = (u8*) &gKeys[40];
    RememberChar = *ptrChar;
    *ptrChar = 0;
    cpct_setDrawCharM1(2, 0);
    pvmem = getScreenLineAdress (19);
    cpct_drawStringM1(gKeys,pvmem);
    *ptrChar = RememberChar;

    ptrChar = (u8*) &gHint[80];
    RememberChar = *ptrChar;
    *ptrChar = 0;
    pvmem = getScreenLineAdress (21);
    cpct_drawStringM1(&gHint[40],pvmem);
    *ptrChar = RememberChar;

    ptrChar = (u8*) &gCopy[40];
    RememberChar = *ptrChar;
    *ptrChar = 0;
    cpct_setDrawCharM1(1, 0);
    pvmem = getScreenLineAdress (24) + 3;
    cpct_drawStringM1(gCopy,pvmem);
    *ptrChar = RememberChar;

    startMusic(MUS_MENU);
}

void displayMenuValues ()
{
    pvmem = getScreenLineAdress (8) + 28;

    strBuffer [0] = '1'+gGame;
    strBuffer [1] = ':';
    cpct_setDrawCharM1(2, 0);

    getStringFromInt(&strBuffer [2], gGridWidth);
    getStringFromInt(&strBuffer [5], gGridHeight);
    strBuffer [5] = 'x';
    strBuffer [8] = ' ';
    getStringFromInt(&strBuffer [9], gGridNbBomb); // It sests the last char to 0
    cpct_drawStringM1(strBuffer,pvmem);

    pvmem = getScreenLineAdress (10) + 44;
    cpct_drawStringM1((gUseAutoStart?strOn:strOff),pvmem);

    pvmem = getScreenLineAdress (12) + 44;
    cpct_drawStringM1( (gUseQMark?strOn:strOff),pvmem);
}

void displayGridInfos()
{
    i16 nbBombsLeft;
    pvmem = getScreenLineAdress (10) + 72;
    cpct_setDrawCharM1(2, 0);
    nbBombsLeft = gGridNbBomb - gNbFlag;
    if (nbBombsLeft>=0)
        getStringFromWord (strBuffer, gGridNbBomb - gNbFlag);
    else
    {
        cpct_setDrawCharM1(3, 0);
        strBuffer[2] = 'E';
        strBuffer[3] = 'R';
        strBuffer[4] = 'R';
        strBuffer[5] = 0;
    }
    cpct_drawStringM1(&strBuffer[2],pvmem);

    pvmem = getScreenLineAdress (12) + 68;
    cpct_setDrawCharM1(2, 0);
    getStringFromWord (strBuffer, gGridSize - gGridNbBomb - gNbDig);
    cpct_drawStringM1(strBuffer,pvmem);
}

void displayClock(u16 clock)
{
    u8 *pvmemIntr = getScreenLineAdress (14) + 72; // Need specific ptr, because called from inside interrupt
    cpct_setDrawCharM1(2, 0);
    getStringFromWord (strBuffer, clock);
    cpct_drawStringM1(&strBuffer[2], pvmemIntr); // Don't mind 2 first digit because capped at 999
}

void displayPlayscreen ()
{
    pvmem = getScreenLineAdress (10) + 64;
    printSprite (pvmem, SPR2_BOMB);
    pvmem+=2;
    cpct_setDrawCharM1(2, 0);
    cpct_drawStringM1("-",pvmem);
    pvmem+=2;
    printSprite (pvmem, SPR2_FLAG);

    pvmem = getScreenLineAdress (12) + 64;
    printSprite (pvmem, SPR2_C0);

    pvmem = getScreenLineAdress (14) + 64;
    strBuffer[0] = 'T';
    strBuffer[1] = ' ';
    strBuffer[2] = ':';
    strBuffer[3] = ' ';
    strBuffer[4] = 0;
    cpct_drawStringM1(strBuffer,pvmem);

    pvmem = getScreenLineAdress (16) + 64;
    getStringFromWord (&strBuffer[2], getHiscoreValue(gGame));
    strBuffer[0] = 'H';
    strBuffer[1] = 'I';
    strBuffer[2] = ':';
    cpct_drawStringM1(strBuffer, pvmem); // Don't mind 2 first digit because capped at 999
}

void displayExit ()
{
    cpct_setDrawCharM1(1, 0);
    pvmem = getScreenLineAdress (24) + 2;
    cpct_drawStringM1(hisc_rstrt,pvmem);
}

void displayFailure ()
{
    stopMusic();
    gShowBomb = 1;
    printGrid ();
    printPointerScreen ();
    cpct_setDrawCharM1(3, 0);
    pvmem = getScreenLineAdress (18) + 64;
    cpct_drawStringM1(sFAILED,pvmem);
    displayExit ();
    startMusic (MUS_EMPTY);
}

u8 displaySuccess ()
{
    u8 rc;
    stopMusic();
    cpct_setDrawCharM1(2, 0);
    pvmem = getScreenLineAdress (18) + 64;
    cpct_drawStringM1(sSUCCESS,pvmem);    
    rc = checkHighScore (gGame,getGameClock());
    if (rc != 1)
    {
        startMusic (MUS_WIN);
        displayExit ();
    }
    else
        startMusic (MUS_HSCORE);

    return rc;
}

void updateMenuAnimation()
{
    // int rndval, x, y;
    // Menu animation because I can
    menuAnimFrameTick++;

    if (menuAnimFrameTick <= 40)  // 40 char to scroll
    {
        menuAnimPos += menuAnimDirection;
        displayAnimMenu (menuAnimPos);
    }
    else if (menuAnimFrameTick == 250)
    {
        menuAnimFrameTick = 0; 
        menuAnimDirection *= -1;
        startExploding = 1;
    }

    // Exploding animation
    if (startExploding)
        updateBombExpl ();
}

void nextMusicOption()
{
    changeMusicOption ();
    displayMusicOption (getMusicOption());
}
