; Code from Shining found in CPCWiki thread: https://www.cpcwiki.eu/forum/programming/save-file-without-firmware/
; CheckTapeOrDisk based on an idea from Jean-Marie on CPCWiki thread https://www.cpcwiki.eu/forum/programming/how-to-check-if-program-was-launched-from-disk-or-tape
; named y_disk to make sure to be after &4000
	.module y_disk
	.optsdcc -mz80

;--------------------------------------------------------
; Exports of this module
;--------------------------------------------------------
; methods
	.globl _CheckTapeOrDisk
	.globl _StoreDriveLetter
	.globl _LoadFile
	.globl _SaveHiScore

;--------------------------------------------------------
; Imports from another modules
;--------------------------------------------------------
; variables
	.globl _sHSFile
	.globl _isTape
  .globl _hiscore

  ; Checks if a filename is set in A70A (Disk) so first byte should be greater equal 64
  ; On tape, the filename is not set and area remains 0 (at least on emulator, to be tested on real hardware)
_CheckTapeOrDisk:
  ; 
  ld hl,#0xa70a
  ld b,#0x0
  ld a,(hl)
  cp #0x40   ; 
  jr nc,_isdisk
  inc b
  _isdisk:
  ld a,b 
  ld (_isTape),a 
  ret

_StoreDriveLetter:
  ;;------------------------------------------------------------------------
  ;; store the drive number the loader was run from to InitializeAmsdos
  ld hl,(#0xbe7d)
  ld a,(hl)                 
  ld (_driveLoad+1),a                 
  
  ;;------------------------------------------------------------------------
  ;; store jump restore call to InitializeAmsdos
  ld a,(#0xbD38)                 
  ld (_firmJump+1),a
   
  ret

_InitializeAmsdos:
  ;; ** INITIALISE ALTERNATE REGISTER SET FOR FIRMWARE **
  exx
  ld bc,#0x7f88     ;initialise lower rom and select mode 0
  out (c),c         ;this routine must be located above &4000
  exx 
  ex af,AF
  xor a
  ex af,af

  ;; ** INITIALISE FIRMWARE **
  call #0x0044      ;initialise lower jumpblock (&0000-&0040)
                  ;and high kernal jumpblock (&b800-&bae4)
  _firmJump: call #0x08bd                ;initialise firmware jumpblock entries
                      ;call #0xBD37
                      ;(&bb00- ...)
  call #0xbb00                ;initialise keyboard manager
  call #0xb909                ;disable lower rom

  ;;; ** INITIALISE DISK ROM FOR LOADING/SAVING **
  ld c,#7                      ;disk rom
  ld de,#0x8000                 ;lowest useable byte of memory
  ld hl,#0xb0ff                 ;highest useable byte of memory
  call #0xbcce           ;initialise disk rom

  ld c,#6                      ;disk rom
  ld de,#0x8000                 ;lowest useable byte of memo ry
  ld hl,#0xb0ff                 ;highest useable byte of memory
  call #0xbcce           ;initialise disk rom

  ld a,#0xc9
  ld (#0xbb5a),a           ;prevent printing of text characters
                      ;don t get error messages corrupting screen

  ld a,#0xff
  ld (#0xbe78),a                ;turn off disc error messages
                    
  ;;------------------------------------------------------------------------
  ;; when AMSDOS is enabled, the drive reverts back to drive 0!
  ;; This will restore the drive number to the drive the loader was run from
  _driveLoad: ld a, #0x00
  ld hl,(#0xbe7d)
  ld (hl),a 

  ret
   
_LoadFile:
  di
  ld (stackLoad+1),sp             ;save stack used by program
  ld sp,#0x9000                 ;new stack used by firmware and load routines

  call _InitializeAmsdos

  ;; ** LOAD FILE USING FIRMWARE **                           

  ;; B = length of the filename in characters
  ld b, #11

  ;; HL = address of the start of the filename
  LD HL, #_sHSFile 

  ;; DE = 2k RAM of buffer
  LD DE, #0x9000

  ;; Read filename and first 2k block
  call #0xbc77 ; cas_in_open

  ld HL, #_hiscore
  call #0xbc83 ;cas_in_direct

  call #0xbc7a ;cas_in_close

  stackLoad: ld sp,#0              ;old stack back and...
  ret                         ;...exit to program!!

_SaveHiScore:
  ld a,(_isTape)
  or a
  ret nz

  di
  ld (stack+1),sp             ;save stack used by program
  ld sp,#0x9000                 ;new stack used by firmware and load routines

  call _InitializeAmsdos
    
  ;; ** SAVE FILE USING FIRMWARE **                           

  ;; B = length of the filename in characters
  ld b, #11
  ;; HL = address of the start of the filename
  LD HL, #_sHSFile 
  LD DE, #0x9000
  call #0xbc8c ; cas_out_open

  ld bc,#0
  ld a,#2  ; BINARY 
  LD DE, #24 ; Size of file   
  ;; HL = address of the start of the filename
  ld HL, #_hiscore
  call #0xbc98 ;cas_out_direct
  call #0xbc8f ;cas_out_close

  ld bc,#0xFA7E            ; FLOPPY MOTOR OFF
  out (c),c               

  stack: ld sp,#0              ;old stack back and...
  ret                         ;...exit to program!!

